
/* Global variables for the Form Filler program */

#include <stdio.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <string.h>
#include <malloc.h>
#include "z100keys.h"
#include "keycodes.h"

#define MALLOC(x)   ((x *)malloc(sizeof(x)))
#define CALLOC(n,x) ((x *)calloc(n,sizeof(x)))
#ifdef DEBUG
#define D(x) x
#else
#define D(x)
#endif
#define MAX_ROWS    600
#define MAX_COLS    132

/* 
	BOT: # of rows at bottom of screen for status line & entry menu

*/
#define BOT         3
#define S_DOWN      0
#define S_UP        1
#define ERASE       0
#define PADCHAR 26   /* ^Z MS-DOS end of ASCII file */
#define SCREEN_LINE '*'
#define SCREEN_LOC  '='
#define CNST_LINE   '@'
#define CNST_LOC    '^'
#define CNST_CTRL   '~'

/* This is the number of the field_keyword modifier which is the last
   type name, the one before Uppercase, RightJustify, etc. */
#define MAX_TYPE_MODIFIER 5

extern int  gedits();
extern void cls();
extern void setcur();
extern void wchar();
extern void wstr();
extern int  gtkey();
extern int  rows;

/* Input Process Structures */

struct   field
{
	char name[10];			/* Name of field */
	char *formula;			/* Formula for field */
	int  entry_row, entry_col;	/* Placement on data entry screen */
	int  print_row, print_col;	/* Placement on printed form */
	int  type;			/* Type of field.  0 if not a field. */
	char *work;			/* Current value of field */
					/* if NULL, either ENTRY_LINE points to
					   data-entry screen text, or
					   MASK points to constant data */
	double value;			/* Value of field, if numeric */
	char *last;			/* Previous value of field */
	char *mask;			/* Mask chars for field */
	char *entry_line;		/* Pointer to data-entry-screen line */
	char prot, reqd, ucaseonly, rightjust;
					/* TRUE if:
					   protected,
					   required,
					   uppercase only,
					   right justified */
					/* ucaseonly also doubles as flag to show
					   that a field is being evaluated, so we
					   don't infinitely recurse! */
	char non_printing;		/* TRUE if we don't print it */
	struct field *next, *prev, *next_print;
					/* Next and previous fields for
					   data entry; and next field for
					   printing */
};


#ifdef MAIN_PROG

#ifdef DEMO
	char program[] = "FORM FILL-R \1Demo\2";
#else
	char program[] = "FORM FILL-R";
#endif
	char version[] = "Version 2.12a";
	char copyright[]="Copyright (c) 1988, Lindley Systems";
	char *LS=copyright+20;

	char WFormDefn[] = "*.FDF";		/* Form definition extension */
	char *FormDefn = WFormDefn + 1;
	
	char WFormSave[] = "*.FIF";		/* Saved form extension */
	char *FormSave = WFormSave + 1;

	char MainMenu[]="Main Menu";

	char Printer[]="PRN";			/* Default print device */
	char Shell[] = "COMMAND";		/* Command Shell */
	char Path[]="PATH";			/* Place to look for files */
	char helpname[66];			/* Name of Help file */

	char ReadBinary[]="rb";

	/* Color attributes */
	int cur_attr;
	int nor_attr=0x07;
	int inv_attr=0x70;

	struct field *first_field = NULL,	/* Pointer to first field */
		     *cur_field = NULL,		/* Field currently being processed */
		     *last_field = NULL,	/* Pointer to last field defined */
		     *first_print_field = NULL,	/* Pointer to first field in print list */
		     *first_data_field = NULL;	/* First field with data */


	int win_sr;		/* Screen row where window starts */
	int win_lr;		/* Last row of window */
	int win_nr;		/* Number of lines in window */
	int cur_top;		/* Line currently at top of window */ 
	int last_line;		/* Last line of user form definition */

	/* Status messages */
	char *stat_line;
	char stat_msg[81];
	char no_form[]="Press F3 or F6 to select a form";
	char empty[]={""};
	char not_found[]="File not found";
	char any_key[]="Press any key to continue...";

	/* Misc. things */
	FILE *prn_file, *form_file;
	char ul_mode = 0;	/* UNDERLINE mode */
	char data_mode = 0;	/* DATA mode */
	char quote_mode = 0;	/* QUOTES mode */
	char screen_mode=0;	/* SCREEN mode */
	char form_feed=1;	/* FF mode */
	char pause_mode=0;	/* PAUSE mode */
	/* get_mode  = 0 use default data on first try, then get user data
		     = 1 get data from user
		     = 3 never get user data
	*/
	int  get_mode=1;
	int  form_file_open=0;	/* Form definition in memory? */
	int  prn_file_open=0;	/* Whether we've selected output file */
	int  debug = 0;		/* /D switch */
	int  ext_input = 0;	/* /X switch */
	int  top_mgn, left_mgn;	/* TOP and LEFT margins */
	int  form_len=0;	/* FORMLENGTH (If FORM_FEED is set and this
				   is nonzero, put out linefeeds to fill this
				   many lines */
	char ch, unit[13];
	char form_title[72];
	char prn_name[65];
	char form_name[65];
	char bgd[31];
	/*
	    TEMPORARY VARIABLES, used while loading:

		temp1	contains form definition file name
		temp2	contains print file name
		temp3	contains filled-in form name

	*/
	char temp1[81], temp2[81], temp3[81];
	char blanks[81];

#ifdef Z100
	char HelpFile[]="ZFFHELP";
	char EnvFile[] ="ZFF.ENV";
	char HelpMenu[]="\1 Down \2 Next Page, \1 Up \2 Previous Page, \1 F10 \2 Exit";
#else
	char HelpFile[]="FFHELP";
	char EnvFile[] ="FF.ENV";
	char HelpMenu[]="\1 PgDn \2 Next Page, \1 PgUp \2 Previous Page, \1 F10 \2 Exit";
#endif

	/* Fatal Error messages */
	char err_mem[]="Out of Memory";

	char *header_keywords[]=
		{"SCREEN","DATA","NOFF","UNDERLINE","LEFT","TOP",
		 "QUOTES","FORMLENGTH","PAUSE","\0"};
	char *field_keywords[]=
		{"TEXT","NUMERIC","INTEGER","DOLLARS","REAL","DATE",
		 "UPPERCASE","RIGHTJUSTIFY","REQUIRED","NAME",
		 "FORMULA","NOPRINT","\0"};
	char *formula_keywords[]=
		{"ABS","AVG","CASE","COUNT","IF","INT","MIN","MAX","\0"};

#else

	extern char program[], version[];

	extern char WFormDefn[], *FormDefn, WFormSave[], *FormSave;
	extern char MainMenu[];
	extern char copyright[],*LS;

	extern char Printer[];
	extern char Shell[];
	extern char Path[];
	extern char helpname[];

	extern char ReadBinary[];

	/* Status messages */
	extern char *stat_line;
	extern char stat_msg[81];
	extern char no_form[];
	extern char empty[];
	extern char not_found[];
	extern char any_key[];
	/* Misc. things */
	extern FILE *prn_file, *form_file;
	extern char ul_mode;
	extern char data_mode;
	extern char quote_mode;
	extern char screen_mode;
	extern char form_feed;
	extern char pause_mode;
	extern int  form_file_open;
	extern int  prn_file_open;
	extern int  debug, ext_input;
	extern int  top_mgn, left_mgn;
	extern int  form_len;
	extern char bgd[];
	extern char temp1[], temp2[], temp3[];
	extern char blanks[];
	extern char form_title[31];
	extern char prn_name[65];
	extern char form_name[65];

	extern struct field *first_field; /* Pointer to first field */
	extern struct field *last_field;  /* Pointer to last field defined */
	extern struct field *cur_field;	  /* Field currently being processed */
	extern struct field *first_print_field;
					/* Pointer to first field in print list */
	extern struct field *first_data_field;

	extern int win_sr;		/* Screen row where window starts */
	extern int win_lr;		/* Last row of window */
	extern int win_nr;		/* Number of lines in window */
	extern int cur_top;		/* Line currently at top of window */ 
	extern int last_line;		/* Last line of user form definition */

	extern int    get_mode;

	extern char HelpFile[];
	extern char EnvFile[];
	extern char HelpMenu[];

	/* Fatal Error messages */
	extern char err_mem[];
	extern char err_hdr[];
	extern char err_ivl[];

	extern char *header_keywords[], *field_keywords[], *formula_keywords[];

#endif
