/* User console BIOS functions for IBM-PC compatible machines.

int    gtkey()
void   cls()
int    getpage()
void   cursize (kind)
short  getcur()
void   scur(row,col, pagenum)
short  gcur(pagenum)
void   setcur(to_row,to_col) 
void   wchar(c)
void   wstr(str, move_cur)
void   scroll(dir,top_row,nr_rows)
*/
#include <stdio.h>
#include <dos.h>
#include "z100keys.h"
#include "keycodes.h"

#define VIDEOROWS 0x00400084L

/* BIOS interrupts */
#define VIDEO_IO	0x10
/* video routine numbers */
/* (placed in register AH before a BIOS interrupt 10H */
#define GET_STATE	15

static char far *vid_mem;
static int bpr, col, cols, row, vmode;
int rows;
void scroll(int r, int c, int t, int d);
void _wchar(int c);
void wchar(int c);

/* #define DEBUG 1  */

#define VIDEO_INT    0x10
#define CUR_SIZE     0x1
#define SET_POSN     0x2
#define READ_POSN    0x3
#define SCROLL_UP    0x6
#define SCROLL_DOWN  0x7
#define WRITE_CHR    0x9
#define WRITE_TTY    0xe
#define GET_VMODE    0xf


/* Cursor values from IBM BIOS */
#define I_LEFT		75
#define I_RIGHT		77
#define I_CTL_LEFT	115
#define I_CTL_RIGHT	116
#define I_PGUP		73
#define I_PGDN		81
#define I_INS		82
#define I_DEL		83
#define I_HOME		71
#define I_END		79
#define I_SH_TAB	15
#define I_CTL_HOME	119
#define I_CTL_END	117
#define I_UP		72
#define I_DOWN		80
#define I_F1		59
#define I_F2		60
#define I_F3		61
#define I_F4		62
#define I_F5		63
#define I_F6		64
#define I_F7		65
#define I_F8		66
#define I_F9		67
#define I_F10		68

#define BDOS_IN      8

extern int    get_mode;
/* Color attributes */
extern int    cur_attr;
extern int    nor_attr;
extern int    inv_attr;

static union  REGS Regs;

/* gtkey ------------------------------------------------------
   Return a key from the keyboard and map control characters */

int gtkey()
{
    register int c;
 
    c=bdos(BDOS_IN,0,0) & 0xff;
    if (!(c)) /* Zero means special function key */
    {   c=bdos(BDOS_IN,0,0) &0xff;
	switch (c)
	{
	case I_LEFT:	  return(LEFT);
	case I_RIGHT:	  return(RIGHT);
	case I_CTL_LEFT:  return(CTL_LEFT);
	case I_CTL_RIGHT: return(CTL_RIGHT);
	case I_INS:	  return(INS);
	case I_DEL:	  return(DEL);
	case I_HOME:	  return(HOME);
	case I_END:	  return(END);
	case I_CTL_HOME:  return(SK7);
	case I_CTL_END:	  return(SK1);
	case I_SH_TAB:	  return(SH_TAB);
	case I_UP:	  return(UP);
	case I_DOWN:	  return(DOWN);
	case I_PGUP:	  return(PGUP);
	case I_PGDN:	  return(PGDN);
	case I_F1:	  return(F1);
	case I_F2:	  return(F2);
	case I_F3:	  return(F3);
	case I_F4:	  return(F4);
	case I_F5:	  return(F5);
	case I_F6:	  return(F6);
	case I_F7:	  return(F7);
	case I_F8:	  return(F8);
	case I_F9:	  return(F9);
	case I_F10:	  return(F10);
	default:	  return(NULL);
	}
    }
    else if (c=='\r')    return('\n');
    else if (c=='\n')    return('\r');
    else                 return(c);
}

/* video_mode--------------------------------------------------
   Returns the current display mode */

int video_mode()
{
    Regs.h.ah=GET_VMODE;
    Regs.h.al=0xFF;			/* Set to illegal value */
    int86(VIDEO_INT, &Regs, &Regs);
    cols = Regs.h.ah;			/* Columns on display (80 usually) */
    return (int) Regs.h.al;
}

/* init_keys ---------------------------------------------------------

   Provided in the Z-100 version to set up the keyboard,
   this actually initializes the display adapter segment number.
*/

void init_keys()
{
    vmode = video_mode();
    if (vmode == 0xFF)		/* Whoa!! Must be Z-100 */
      { fprintf(stderr,"Unknown Video Mode.");
	exit(9);
      }
    bpr = cols--*2;		/* Bytes per Row */
				/* Decrement cols to be 0..79 */
    vid_mem = (vmode == 7) ? (char far *)0xB0000000L : (char far *)0xB8000000L;
    row = col = 0;

	rows = (int)(*(char far *)VIDEOROWS);
	if (rows == 0)
	  rows = 24;
}

/* exit_keys ---------------------------------------------------------
	Restores the screen color to black & white.
	Called just before the application exits.
*/

void exit_keys()

{
   nor_attr = 07; wchar('\2');
}

/* getpage -----------------------------------------------------
   Returns the currently active display page */

int getpage()
{
    Regs.h.ah=GET_VMODE;
    int86(VIDEO_INT, &Regs, &Regs);
    return (int) Regs.h.bh;
}

/* cursize -----------------------------------------------------
   sets the height of the cursor */

void cursize (kind)
int kind;
{
    int top_line, bot_line;

    bot_line = (vmode == 7)?12:7;
	/* 12 scan lines in monochrome, 7 in color */
    top_line = (kind)?0:bot_line-1;
	/* Full cursor in insert, two-scan-line tall elsewise */

    Regs.h.ch=top_line;
    Regs.h.cl=bot_line;
    Regs.h.ah=CUR_SIZE;
    int86(VIDEO_INT, &Regs, &Regs);
}

/* scur --------------------------------------------------------
   Set cursor to the given position.
*/
void scur(rw, cl, pagenum)
short rw,cl;  int pagenum;
{
    Regs.h.dh=rw;
    Regs.h.dl=cl;
    Regs.h.bh=pagenum;
    Regs.h.ah=SET_POSN;
    int86(VIDEO_INT, &Regs, &Regs);
}

/* gcur --------------------------------------------------------
   Gets the current cursor position */

short gcur(pagenum)
int pagenum;
{
    Regs.h.bh=pagenum;
    Regs.h.ah=READ_POSN;
    int86(VIDEO_INT, &Regs, &Regs);
    return (Regs.x.dx);
}

/* setcur ------------------------------------------------------
   Sets the cursor to the given position */

void setcur(to_row,to_col) 
int to_row, to_col;

{  if (get_mode == 3) return;
    scur(to_row,to_col,getpage());
    row = to_row; col = to_col;
}

/* cls ---------------------------------------------------------
   Clears the console screen */

void cls ()

{
    Regs.h.ch=0;
    Regs.h.cl=0;
    Regs.h.dh=rows;
    Regs.h.dl=cols;
    Regs.h.bh=cur_attr;
    Regs.h.ah=SCROLL_UP;
    Regs.h.al=0;
    int86(VIDEO_INT, &Regs, &Regs);
    setcur(0,0);
}

/* getcur ------------------------------------------------------
   Gets the current cursor position */

short getcur()
{
   return gcur(getpage());
}

/* _wchar -------------------------------------------------------
	Writes one character to screen.
	Does Not move the cursor, though it does update the
	current row and column variables.
*/

void _wchar(c)
int c;
{   char far *v;

    switch (c)
    {
	case '\01':
	    cur_attr=inv_attr;
	    break;
	case '\02':
	    cur_attr=nor_attr;
	    break;
	case BEL:  /*  Bell */
	    Regs.h.bl=0;
	    Regs.h.al='\07';
	    Regs.h.ah=WRITE_TTY;
	    int86(VIDEO_INT, &Regs, &Regs);
	    break;
	case BS:  /* Backspace */
	    if (col > 0) --col;
	    break;
	case NL:  /* Line feed */
	    if (row < rows)
		++row;
	    else
		scroll(0,rows,1,1);
	    break;
	case CR:  /* Carriage return */
	    col = 0;
	    break;
	default:
	/* Show character with attributes. Writes directly to Video RAM */
	    if (col <= cols)		/* Only display if on screen */
	      {	v = vid_mem;
		v += (bpr * row) + (col << 1);
		*v++ = c;
		*v   = cur_attr;
	      }
	    col++;
    }
}

/* wchar --------------------------------------------------------
   Writes a character to screen.
   Moves the cursor.
*/

void wchar (int c)
{
   if (get_mode != 3)
     {	_wchar(c);
	setcur(row,col);
     }
}

/* wstr --------------------------------------------------------
   Writes a string to screen */

void wstr(str, move_cur)
char *str;
int move_cur;
{  register short posn;
   if (get_mode == 3) return;
   if (!move_cur) posn=getcur();
   while (*str) _wchar(*str++);		/* _wchar DOES NOT!! move the cursor */
   if (move_cur)
	setcur(row,col);		/* Move the cursor if we are */
   else
     {	row = posn/256;			/* else just reset the variables */
	col = posn%256;
     }
}

/*
	Erase a region of the screen.
	Does not move the cursor.
	If -1 is specified as the minimum row, the
	current cursor line will be used.
*/

void erase (min_row, max_row, min_col, max_col)
    int min_row, max_row, min_col, max_col;
{   int posn;

    posn=getcur();			/* Save current posn */

    if (min_row < 0)
      min_row = max_row = posn/256;	/* Use current cursor line */

    for (row=min_row; row <= max_row; row++)
      for (col=min_col; col <= max_col; )
	_wchar(32);

    row = posn>>8;			/* reset row & col */
    col = posn%256;
}

/* clear_eol --------------------------------------------------
   Clears to the end of the line */

void clear_eol()
{  register short posn;

	if (get_mode == 3) return;
	posn=getcur();			/* Save current posn */

	row = posn>>8;			/* reset row & col */
	col = posn%256;

	erase(row,row,col,cols);	/* Write spaces to end of line */
}

/*---------------------------------- scroll ---------------------
    Scrolls up/down the given number of lines from top line
    dir = 1 means scroll up, 0 means scroll down
*/
void scroll(top_row,bot_row,to_do,dir)
int         top_row,bot_row,to_do,dir;
{
    if (dir)  
	Regs.h.ah=SCROLL_UP;
    else
	Regs.h.ah=SCROLL_DOWN;
    Regs.h.al=to_do;
    Regs.h.ch=top_row;
    Regs.h.cl=0;
    Regs.h.dh=bot_row;
    Regs.h.dl=cols;
    Regs.h.bh=nor_attr;
    int86(VIDEO_INT, &Regs, &Regs);
}
/* test main ---------------------------------------------------
   */
#if defined (DEBUG)
main()
{  int r,c;

   cls();
   setcur(5,15);
   wstr("This is a test of cls, setcur, and wstr",1);
   getchar();
   setcur(7,20);
   wstr("This is the second line of the test",0);
   getchar();
   setcur(9,25);
   wstr("This is the third line of the test",1);
   getchar();
}
#endif
